@@ -1,6 +1,7 @@
 import asyncio
 import json
 import re
+import sys
 
 import aiohttp
 import async_timeout
@@ -39,7 +40,7 @@ class Receiver():
             headers: HTTP请求头，用于post请求，需要包含cookie信息。
 
         """
-        self.Record = {'score': 0, 'values': []}
+        self.Score = 0
         self.Headers = headers
         self.EnableURL = "https://api.live.bilibili.com/av/v1/SuperChat/enable?room_id=%s&parent_area_id=%s&area_id=%s&ruid=%s"
         self.PostURL = "https://api.live.bilibili.com/xlive/lottery-interface/v3/guard/join"
@@ -51,23 +52,35 @@ class Receiver():
         except:
             raise KeyError
 
-    def Start(self, roster: Roster, timeout=1):
+    def Start(self, roster: Roster, timeout=1, merge=1, delay=0, proxy=None, log=sys.stdout):
         """ 统一启动接口
         接口接受一个加载完毕的Base.Roster对象作为参数，并依附于其的事件循环，创建任务并管理日志输出。
         因为B站有舰长抽奖的直播间不会太多，直播高峰期也不过近百，所以这里为每个直播间都申请一个任务。
 
         Args: 
-            roster:  一个Base.Roster对象。
-            timeout:    超时限制，超过此值会post失败并在日志中加入Time out error信息，默认为1s。
+            roster:     一个Base.Roster对象。
+            timeout:    超时限制，超过此值会post失败并输出日志。
+            merge:      任务的串行程度，数值越低并发的程度越低，效率越高，一般可以适当提高数字增加运行时间。
+            delay:      请求间的延迟，和merge配合降低运行效率防止IP被ban。
+            proxy:      使用代理，默认不使用。
+            log:        日志输出目标，为文件对象，默认为sys.stdout。
         """
         taskList = []
+        # '''
+        i = 0
+        while i < len(roster.LiveRoomList):
+            taskList.append(roster.EventLoop.create_task(
+                self.LinearReceive(roster.LiveRoomList, i, timeout, merge, delay, proxy, log)))
+            i += merge
+        '''
         for liverMsg in roster.LiveRoomList:
             taskList.append(roster.EventLoop.create_task(
-                self.Receive(liverMsg, timeout)))
+                self.Receive(liverMsg, timeout, proxy=proxy, log=log)))
+        '''
         roster.EventLoop.run_until_complete(asyncio.wait(taskList))
 
-    async def Receive(self, liverMsg: dict, timeout):
-        """ 异步HTTP请求的亲密度领取接口
+    async def Receive(self, liverMsg: dict, timeout, proxy, log):
+        """ 异步亲密度领取接口
         此接口领取一个直播间的所有抽奖。
         按照b站的API来看，需要先请求一次权限，第二次请求才能得到抽奖信息的id，接着再通过id来发出post请求
         post请求设有csrf防御，需要用到csrf_token。
@@ -98,8 +111,8 @@ class Receiver():
                         senderList = (json.loads(await checkRes.text())).get('data').get('guard')
                         # print(json.dumps(senderList, indent=2))
                 else:
-                    Base.addRecordMsg(
-                        self.Record, roomid, "Bad get.", -1)
+                    Base.log(
+                        log, self.self.Score, roomid, "Bad get.", -1)
                     return
             for msg in senderList:
                 # 生成请求数据
@@ -118,34 +131,38 @@ class Receiver():
                     with async_timeout.timeout(timeout):
                         async with session.post(self.PostURL, data=data, headers=self.Headers) as res:
                             if res.status == 200:
+                                await asyncio.sleep(0.1)
                                 response = json.loads(await res.text())
                                 # code=0 时，代表领取成功
                                 if response.get('code') == 0:
-                                    Base.addRecordMsg(self.Record, roomid, "Successful received.", int(
-                                        response.get('data').get('award_num')))
+                                    score = int(response.get(
+                                        'data').get('award_num'))
+                                    self.Score += score
+                                    Base.log(log, self.Score, roomid,
+                                             "Successful received.", score)
                                 # code=400 时，代表已领取
+                                elif response.get('code') == 400:
+                                    Base.log(log, self.Score, roomid,
+                                             "You already received.", 0)
                                 else:
-                                    Base.addRecordMsg(self.Record, roomid,
-                                                      "You already received.", 0)
+                                    Base.log(log, self.Score, roomid,
+                                             response.get('message'), -1)
                             else:
                                 # 请求失败时把信息抛到序列尾择期执行。
-                                Base.addRecordMsg(
-                                    self.Record, roomid, "Bad post and try again.", -1)
+                                Base.log(
+                                    log, self.Score, roomid, "Bad post and try again.", -1)
                                 senderList.append(msg)
                                 continue
                 except asyncio.TimeoutError as e:
-                    # senderList.append(msg)
-                    Base.addRecordMsg(
-                        self.Record, roomid, "Time out error.", -1)
+                    Base.log(
+                        log, self.Score, roomid, "Time out error.", -1)
                     continue
 
-    def Clear(self):
-        """ 清除日志(仅日志信息)
-        """
-        self.Record.get('values').clear()
-
-    def ClearAll(self):
-        """ 清除日志(包括领取值)
-        """
-        self.Record.get('values').clear()
-        self.Record['score'] = 0
+    async def LinearReceive(self, liveRoomList: list, base: int, timeout, merge, delay, proxy, log):
+        top = base+merge
+        if top >= len(liveRoomList):
+            top = len(liveRoomList)
+        for i in range(base, top-1):
+            await self.Receive(liveRoomList[i], timeout, proxy, log)
+            await asyncio.sleep(delay)
+        await self.Receive(liveRoomList[top-1], timeout, proxy, log)
